'' Sample program demonstrating 2D Rigid Body Dynamics

Option Explicit

#include "rigid_types.bi"
#include "fbgfx.bi"
#include "cons_log.bi"
#include "rigid_world.bi"
#include "rigid_engine.bi"
#include "rigid_render.bi"
#include "rigid_scene.bi"

enum ToolModeEnum
	TM_None = 0
	TM_Elastic
	TM_Force
end enum

dim shared as integer restart = True

sub SimulationLoop

	dim as integer mx, my, mw, mb, mb1, mb2, oldmb, oldmb1, oldmb2, oldmw, bShowDesc = 0
	dim as integer i, j, k, objectindex
	dim as real f
	dim as double t1, t2, dt, dt2, tKillShowDesc
	dim as string msg
	dim as vector relpnt, ptmp, p, t
	dim as spring_t ptr spring
	dim as integer ToolMode = TM_None '' ENUM:ToolModeEnum
	
	STATE1 = 0
	STATE2 = 1

	t1 = timer
	spring = 0 '' NULL
	Restart = False

	if SCENE_DESC_TIMER > 0 then
		bShowDesc = 2
		tKillShowDesc = t1 + SCENE_DESC_TIMER 
	else
		SCENE_DESC_TIMER = 15
	end if

	do
		t2 = timer
		dt = t2 - t1
		t1 = t2

		oldmb = mb
		oldmw = mw
		oldmb1 = mb1
		oldmb2 = mb2
		GetMouse mx, my, mw, mb
		mb1 = (mb and 1)
		mb2 = (mb and 2)
		if mx >= 0 then


			'' Make mouse coords a point
			VSet ptmp,csng(mx),csng(my)
			MapScreenPointToWorld p, ptmp
			
			If ToolMode = TM_None then
				'' detect left mouse button click
				if oldmb1 = 0 and mb1 <> 0 then
					'' See if the point is in an object, and capture the point
					objectindex = FindObjectFromPoint(STATE2, p, relpnt)
					if objectindex > 0 then
						spring = Create_Spring()
						spring->springKs = DEFAULT_SPRING_KS
						spring->springKd = DEFAULT_SPRING_KD
						spring->objectindex1 = objectindex
						spring->pnt1 = relpnt
						spring->objectindex2 = 0
						ToolMode = TM_Elastic
					end if
	
				'' detect right mouse button click
				elseif oldmb2 = 0 and mb2 <> 0 then
					'' See if the point is in an object, and capture the point
					objectindex = FindObjectFromPoint(STATE2, p, relpnt)
					if objectindex > 0 then
						ToolMode = TM_Force
					end if
				end if
			end if

			If ToolMode = TM_Elastic then
				'' Left mouse button down updates position
				if mb1 <> 0 then
					if (spring) then
						spring->pnt2 = p				
					end if
				end if

				'' Right mouse click button cancels 
				if oldmb2 = 0 and mb2 <> 0 then
					if (spring) then
						spring->Alive = False
					end if
					ToolMode = TM_None
				end if

				'' detect left mouse button release
				if oldmb1 <> 0 and mb1 = 0 then
					if (spring) then
						'' See if the point is in an object, and capture the point
						objectindex = FindObjectFromPoint(STATE2, p, relpnt)
						if objectindex <> 0 then
							if objectindex <> spring->objectindex1 then
								spring->objectindex2 = objectindex
								spring->pnt2 = relpnt
							else
								spring->Alive = False
							end if
							objectindex = 0
						end if
						spring = 0
					end if			
					ToolMode = TM_None
				end if
			end if

			if ToolMode = TM_Force then
				'' Right mouse button down updates position
				if mb2 <> 0 then
					if objectindex > 0 then
						ApplyForceToObject objectindex, p, (oldmw - mw) * -10
					end if
				end if

				'' detect right mouse button release
				if oldmb2 <> 0 and mb2 = 0 then
					if objectindex > 0 then
						VSet objects(objectindex)->appliedforce, 0, 0
						objects(objectindex)->appliedtorque = 0
						objectindex = 0
					end if
					ToolMode = TM_None
				end if
			end if

		end if

		'' render the frame
		cls
		'' print "In world : " & str(CheckIsInWorld(p))
		if bShowDesc then
			if t1 > tKillShowDesc then
				bShowDesc = 0
			else
				print SCENE_TITLE
				print
				for i = 1 to SCENE_DESC_COUNT
					print SCENE_DESC(i)
				next i
			end if
		end if
		
		dt2 = RenderFrame(dt)
		RenderWorldToScreen STATE2
		
		'screensync
		flip

		if MultiKey(sc_left) then
			screen_center.x -= 240 * dt / screen_mag
		end if
		if MultiKey(sc_right) then
			screen_center.x += 240 * dt / screen_mag
		end if
		if MultiKey(sc_down) then
			screen_center.y -= 240 * dt / screen_mag
		end if
		if MultiKey(sc_up) then
			screen_center.y += 240 * dt / screen_mag
		end if
		if Multikey(sc_pageup) then
			screen_mag += dt
		end if
		if Multikey(sc_pagedown) then
			screen_mag -= dt
		end if
		if multikey(sc_home) then
			screen_center.x = 0
			screen_center.y = 240
			screen_mag = 1.0
		end if
		if multikey(sc_escape) then
			Restart = False
			exit do
		end if
		if multikey(sc_delete) then
			Destroy_All_Springs()
		end if
		if multikey(sc_R) then
			while multikey(sc_R)
				sleep 15
			wend
			Restart = True
			exit do
		end if
		while multikey(sc_P)
			sleep 15
		wend
		if multikey(sc_F1) then
			bShowDesc or= 1
			tKillShowDesc = t1 + SCENE_DESC_TIMER 
		elseif (bShowDesc and 1) then
			bShowDesc xor= 3			
		end if
	loop

	Destroy_All_Springs()
	Destroy_All_Objects()
	Destroy_All_Shapes()

end sub

'' ==========================================================================
'' MAIN
'' ==========================================================================

dim as integer i
dim as string msg

Log_Enable("")
'' Dbg_Enable()
Log_Print( "Program started at " & date() & " " & time() )

randomize timer
screen 18, 16, 2
screenset 1, 0
Randomize Timer
SetScreenMag 1.0
SetScreenCenter 0, 240

while restart = True

	Create_World()

	If (command(1)) > "" then
		i = 1
		do while command(i) > ""
			if LoadScene(command(i)) <> 0 then
				exit while
			end if
			i += 1
		loop
	else
		if LoadScene("default.ini") <> 0 then
			exit while
		end if
	end if

	'' DumpWorldInfo()

	SimulationLoop

	Destroy_World()

wend

Log_Print( "Program ended at " & date() & " " & time() )
Log_Disable()

end
