#ifndef __RIGID_BSP_BI__
#define __RIGID_BSP_BI__

#include "vector.bi"

Const BSP_SOLID = 1
Const BSP_NOTSOLID = 2
Const BSP_FORCE = 4
Const BSP_INVISIBLE = 8

type bsp_node_t
	id as integer
	linedef_index as integer	'' index to linedef for the partition on this bsp node
	perpline1 as integer      '' index to linedef for left side limit
	perpline2 as integer      '' index to linedef for right side limit
	parent as bsp_node_t ptr 	'' parent node						(0=none)
	front as bsp_node_t ptr		'' pointer to front node  (0=none)
	back as bsp_node_t ptr		'' pointer to back node   (0=none)
	flags as integer          '' bsp node flags
	dirindex as integer     '' index to force normal for the region
end type

type bsp_tree_t
	maxpoints as integer        '' maxpoints allocated
	npoints as integer          '' number of points in this tree
	points as vector ptr        '' pointer to a list of points
	maxlinedefs as integer      '' maxlinedefs allocated
	nlinedefs as integer        '' number of linedefs in this tree
	linedefs as line_t ptr      '' pointer to a list of line defs
	root as bsp_node_t ptr      '' pointer to the root node
end type

enum LineDefTypeEnum
  LT_COINCIDENT
  LT_IN_BACK_OF
  LT_IN_FRONT_OF
  LT_SPANNING
end enum

declare function create_bsp_tree() as bsp_tree_t ptr
declare function partition_bsp(byval tree as bsp_tree_t ptr, a as vector, b as vector, r as line_t, flags as integer, dr as line_t) as integer
declare sub delete_bsp_tree(byval tree as bsp_tree_t ptr)
declare function bsp_get_side(byval tree as bsp_tree_t ptr, a as vector) as integer
declare function bsp_get_force(byval tree as bsp_tree_t ptr, a as vector, f as vector) as integer
declare function bsp_find_collision(byval tree as bsp_tree_t ptr, a as vector, d as real, n as vector) as integer
declare sub bsp_dump_tree(byval tree as bsp_tree_t ptr)
declare function bsp_check_visibility(byval tree as bsp_tree_t ptr, byval node as bsp_node_t ptr, a as vector, b as vector) as integer

#endif
