#ifndef __RIGID_ENGINE_BI__
#define __RIGID_ENGINE_BI__

#include "vector.bi"
#include "rigid_shapes.bi"

type objectstate_t
	wpoints as vector ptr     '' pointer to list of vectors in world space
	p as vector								'' position [m, m]
	v as vector								'' velocity [m/s, m/s]
	''a as vector								'' acceleration [m/s2, m/s2]
	angp as real						'' angular position [radians]
	angv as real						'' angular velocity [radians/s]
	''anga as real						'' angular acceleration [radians/s2]
	force as vector						'' force at center of mass
	torque as real					'' torque about center of mass
	'' cached values - valid only after rendered to world
	cs as real							'' cached cos(angp)
	sn as real							'' cached sin(angp)
end type

type object_t
	shp as shape_t ptr
	mass as real						'' mass [Kg]
	moi as real							'' moment of impulse [Kg * m2]
	cor as real             '' coefficient of restitution
	state(0 to 1) as objectstate_t
	appliedforce as vector
	appliedtorque as real

	colcount as integer
	colworld as integer
	colobject as integer

	immovable as integer
	unresolved as integer
	
end type

type spring_t
	Alive as integer
	SpringKs as real
	SpringKd as real

	objectindex1 as integer
	pnt1 as vector
	wpnt1 as vector

	objectindex2 as integer
	pnt2 as vector
	wpnt2 as vector
end type

enum CollisionStateEnum
	CollisionState_None = 0
	CollisionState_Yes = 1
	CollisionState_TooFar = 2
end enum

enum CollisionTypeEnum
	CollisionType_ObjectToWorld = 0
	CollisionType_WorldToObject = 1
	CollisionType_ObjectToObject = 2
end enum

type collision_state_t
	count as integer
	state as integer          '' ENUM:CollisionStateEnum
	unresolved as integer
end type

type collision_t
	state as integer          '' ENUM:CollisionStateEnum
	kind as integer           '' ENUM:CollisionTypeEnum

	objectindex1 as integer   '' object containing the collision point
	pointindex1 as integer    '' point index in the object/object that got the hit

	objectindex2 as integer   '' object containing the collision surface
	                          '' -1 indicates a world linedef

	normal as vector          '' normal of face/wall in collision
end type

declare function Create_object() as object_t ptr
declare function Add_Object(byval obj as object_t ptr) as integer
declare function FindObjectFromPoint(byval stateindex as integer, v as vector, relpnt as vector) as integer
declare function IsPointInObject(byval stateindex as integer, byval objectindex as integer, v as vector, relpnt as vector) as integer
declare function IsPointNearObject(byval stateindex as integer, byval objectindex as integer, v as vector) as integer
declare sub RenderObjectsToWorld(byval stateindex as integer)
declare function RenderFrame(byval dt as double ) as double
declare sub Destroy_All_Objects()
declare sub Destroy_All_Springs()
declare sub ApplyForceToObject(byval objectindex as integer, p as vector, byval t as real)
declare function Create_Spring() as spring_t ptr

extern NUM_OBJECTS as integer
extern objects() as object_t ptr
extern AGRAVITY as vector
extern STATE1 as integer
extern STATE2 as integer
extern collisionstate as collision_state_t
extern collisions() as collision_t
extern NUM_SPRINGS as integer
extern springs() as spring_t ptr
extern DEFAULT_SPRING_KS as real
extern DEFAULT_SPRING_KD as real
extern DEFAULT_COR as real

#endif
