Option Explicit

#include "rigid_types.bi"
#include "rigid_world.bi"
#include "rigid_shapes.bi"
#include "rigid_engine.bi"
#include "rigid_render.bi"

'' ==========================================================================

dim shared screen_center as vector
dim shared screen_mag as real

sub SetScreenCenter( byval x as real, byval y as real )
	VSet screen_center, x, y
end sub

sub SetScreenMag( m as real )
	screen_mag = m
end sub

'' map a world point to screen coordinates
sub MapWorldPointToScreen(v as vector, r as vector)
	v.x = 320 + (r.x - screen_center.x) * screen_mag
	v.y = 240 - (r.y - screen_center.y) * screen_mag
end sub

sub MapScreenPointToWorld(v as vector, r as vector)
	v.x = (r.x - 320) / screen_mag + screen_center.x
	v.y = (240 - r.y) / screen_mag + screen_center.y
end sub

sub RenderLineToScreen(p1 as vector, p2 as vector, byval clr as integer)
	dim v as vector
	MapWorldPointToScreen v, p1
	pset ( v.x, v.y ), clr
	MapWorldPointToScreen v, p2
	line -( v.x, v.y ), clr
end sub

'' draw a shape defined in world space to the screen using the
'' specified object state
sub RenderShapeToScreen(byval shp as shape_t ptr, byval objst as objectstate_t ptr)
	dim i as integer, j as integer, v as vector, nply as integer
	dim ply as polygon_t ptr
	nply = GetPolyCount(shp)
	for i = 0 to nply - 1
		ply = GetPoly(shp, i)
		if ply->npoints > 0 then
			MapWorldPointToScreen v, objst->wpoints[ply->points[0]]
			pset ( v.x, v.y ), ply->clr
			for j = ply->npoints - 1 to 0 step - 1
				MapWorldPointToScreen v, objst->wpoints[ply->points[j]]
				line -( v.x, v.y ), ply->clr
			next j
		end if
	next i
end sub

''draw a wall to the screen
sub RenderWallToScreen(byval wall as wall_t ptr)
	dim v as vector
	MapWorldPointToScreen v, worldpoints(wall->aindex)
	pset ( v.x, v.y ), rgb(255,255,255)
	MapWorldPointToScreen v, worldpoints(wall->bindex)
	line -( v.x, v.y ), rgb(255,255,255)
end sub

'' draw all shapes in the world to the screen
sub RenderWorldToScreen(byval stateindex as integer)
	dim i as integer
	for i = 1 to NUM_WALLS
		RenderWallToScreen walls(i)		
	next i
	for i = 1 to NUM_OBJECTS
		RenderShapeToScreen objects(i)->shp, @objects(i)->state(stateindex)
	next i
	for i = 1 to NUM_SPRINGS
		if springs(i)->Alive then
			RenderLineToScreen springs(i)->wPnt1, springs(i)->wPnt2, rgb(127,127,127)
		end if
	next i

#if 0
	'' Print some info on the first two objects if debugging

	with objects(1)->state(stateindex)
	print "Position", .p.x, .p.y
	print "Velocity", .v.x, .v.y
	print "Angle", .angp, .angv
	end with

	with objects(1)->state(1 - stateindex)
	print "Position", .p.x, .p.y
	print "Velocity", .v.x, .v.y
	print "Angle", .angp, .angv
	end with

	with objects(2)->state(stateindex)
	print "Position", .p.x, .p.y
	print "Velocity", .v.x, .v.y
	print "Angle", .angp, .angv
	end with

	with objects(2)->state(1 - stateindex)
	print "Position", .p.x, .p.y
	print "Velocity", .v.x, .v.y
	print "Angle", .angp, .angv
	end with

#endif

end sub

